@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.493 2024/05/18 11:17:30 jsg Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.496 2024/09/04 15:06:36 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -33,6 +33,8 @@
 #include <imsg.h>
 
 #define	BGP_VERSION			4
+#define	RTR_MAX_VERSION			2
+#define	RTR_DEFAULT_VERSION		1
 #define	BGP_PORT			179
 #define	RTR_PORT			323
 #define	PEER_DESCR_LEN			64
@@ -109,6 +111,7 @@
 #define	F_CTL_AVS_VALID		0x1000000
 #define	F_CTL_AVS_INVALID	0x2000000
 #define	F_CTL_AVS_UNKNOWN	0x4000000
+#define	F_CTL_FILTERED		0x8000000	/* only set on requests */
 #define	F_CTL_SSV		0x80000000	/* only used by bgpctl */
 
 #define CTASSERT(x)	extern char  _ctassert[(x) ? 1 : -1 ] \
@@ -325,6 +328,7 @@ struct bgpd_config {
 	uint16_t				 min_holdtime;
 	uint16_t				 connectretry;
 	uint8_t					 fib_priority;
+	uint8_t					 filtered_in_locrib;
 };
 
 extern int cmd_opts;
@@ -580,11 +584,19 @@ struct rtr_config {
 	struct bgpd_addr		local_addr;
 	uint32_t			id;
 	uint16_t			remote_port;
+	uint8_t				min_version;
+};
+
+struct rtr_config_msg {
+	char				descr[PEER_DESCR_LEN];
+	uint8_t				min_version;
 };
 
 struct ctl_show_rtr {
 	char			descr[PEER_DESCR_LEN];
 	char			state[PEER_DESCR_LEN];
+	char			last_sent_msg[REASON_LEN];
+	char			last_recv_msg[REASON_LEN];
 	struct bgpd_addr	remote_addr;
 	struct bgpd_addr	local_addr;
 	uint32_t		serial;
@@ -592,12 +604,11 @@ struct ctl_show_rtr {
 	uint32_t		retry;
 	uint32_t		expire;
 	int			session_id;
-	uint16_t		remote_port;
-	uint8_t			version;
 	enum rtr_error		last_sent_error;
 	enum rtr_error		last_recv_error;
-	char			last_sent_msg[REASON_LEN];
-	char			last_recv_msg[REASON_LEN];
+	uint16_t		remote_port;
+	uint8_t			version;
+	uint8_t			min_version;
 };
 
 enum imsg_type {
@@ -889,6 +900,7 @@ struct ctl_neighbor {
 #define	F_PREF_OTC_LEAK	0x080
 #define	F_PREF_ECMP	0x100
 #define	F_PREF_AS_WIDE	0x200
+#define	F_PREF_FILTERED	0x400
 
 struct ctl_show_rib {
 	struct bgpd_addr	true_nexthop;
@@ -1414,6 +1426,7 @@ void		 send_imsg_session(int, pid_t, void *, uint16_t);
 int		 send_network(int, struct network_config *,
 		    struct filter_set_head *);
 int		 bgpd_oknexthop(struct kroute_full *);
+int		 bgpd_has_bgpnh(void);
 void		 set_pollfd(struct pollfd *, struct imsgbuf *);
 int		 handle_pollfd(struct pollfd *, struct imsgbuf *);
 
