@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.h,v 1.303 2024/05/29 10:36:32 claudio Exp $ */
+/*	$OpenBSD: rde.h,v 1.305 2024/08/28 13:21:39 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org> and
@@ -281,19 +281,21 @@ struct prefix {
 	time_t				 lastchange;
 	uint32_t			 path_id;
 	uint32_t			 path_id_tx;
+	uint16_t			 flags;
 	uint8_t				 validation_state;
 	uint8_t				 nhflags;
 	int8_t				 dmetric;	/* decision metric */
-	uint8_t				 flags;
-#define	PREFIX_FLAG_WITHDRAW	0x01	/* enqueued on withdraw queue */
-#define	PREFIX_FLAG_UPDATE	0x02	/* enqueued on update queue */
-#define	PREFIX_FLAG_DEAD	0x04	/* locked but removed */
-#define	PREFIX_FLAG_STALE	0x08	/* stale entry (graceful reload) */
-#define	PREFIX_FLAG_MASK	0x0f	/* mask for the prefix types */
-#define	PREFIX_FLAG_ADJOUT	0x10	/* prefix is in the adj-out rib */
-#define	PREFIX_FLAG_EOR		0x20	/* prefix is EoR */
-#define	PREFIX_NEXTHOP_LINKED	0x40	/* prefix is linked onto nexthop list */
-#define	PREFIX_FLAG_LOCKED	0x80	/* locked by rib walker */
+};
+#define	PREFIX_FLAG_WITHDRAW	0x0001	/* enqueued on withdraw queue */
+#define	PREFIX_FLAG_UPDATE	0x0002	/* enqueued on update queue */
+#define	PREFIX_FLAG_DEAD	0x0004	/* locked but removed */
+#define	PREFIX_FLAG_STALE	0x0008	/* stale entry (graceful reload) */
+#define	PREFIX_FLAG_MASK	0x000f	/* mask for the prefix types */
+#define	PREFIX_FLAG_ADJOUT	0x0010	/* prefix is in the adj-out rib */
+#define	PREFIX_FLAG_EOR		0x0020	/* prefix is EoR */
+#define	PREFIX_NEXTHOP_LINKED	0x0040	/* prefix is linked onto nexthop list */
+#define	PREFIX_FLAG_LOCKED	0x0080	/* locked by rib walker */
+#define	PREFIX_FLAG_FILTERED	0x0100	/* prefix is filtered (ineligible) */
 
 #define	PREFIX_DMETRIC_NONE	0
 #define	PREFIX_DMETRIC_INVALID	1
@@ -301,7 +303,6 @@ struct prefix {
 #define	PREFIX_DMETRIC_AS_WIDE	3
 #define	PREFIX_DMETRIC_ECMP	4
 #define	PREFIX_DMETRIC_BEST	5
-};
 
 /* possible states for nhflags */
 #define	NEXTHOP_SELF		0x01
@@ -378,6 +379,12 @@ int		 peer_imsg_pop(struct rde_peer *, struct imsg *);
 int		 peer_imsg_pending(void);
 void		 peer_imsg_flush(struct rde_peer *);
 
+static inline int
+peer_is_up(struct rde_peer *peer)
+{
+	return (peer->state == PEER_UP);
+}
+
 RB_PROTOTYPE(peer_tree, rde_peer, entry, peer_cmp);
 
 /* rde_attr.c */
@@ -579,7 +586,8 @@ struct prefix	*prefix_adjout_lookup(struct rde_peer *, struct bgpd_addr *,
 		    int);
 struct prefix	*prefix_adjout_match(struct rde_peer *, struct bgpd_addr *);
 int		 prefix_update(struct rib *, struct rde_peer *, uint32_t,
-		    uint32_t, struct filterstate *, struct bgpd_addr *, int);
+		    uint32_t, struct filterstate *, int, struct bgpd_addr *,
+		    int);
 int		 prefix_withdraw(struct rib *, struct rde_peer *, uint32_t,
 		    struct bgpd_addr *, int);
 int		 prefix_flowspec_update(struct rde_peer *, struct filterstate *,
@@ -669,6 +677,12 @@ prefix_re(struct prefix *p)
 	return (p->entry.list.re);
 }
 
+static inline int
+prefix_filtered(struct prefix *p)
+{
+	return ((p->flags & PREFIX_FLAG_FILTERED) != 0);
+}
+
 void		 nexthop_shutdown(void);
 int		 nexthop_pending(void);
 void		 nexthop_runner(void);
