@@ -1,4 +1,4 @@
-/*	$OpenBSD: rtr_proto.c,v 1.35 2024/04/09 12:09:20 claudio Exp $ */
+/*	$OpenBSD: rtr_proto.c,v 1.40 2024/09/10 08:41:13 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -31,11 +31,17 @@
 struct rtr_header {
 	uint8_t		version;
 	uint8_t		type;
-	uint16_t	session_id; /* or error code */
+	union {
+		uint16_t	session_id;
+		uint16_t	errcode;
+		struct {
+			uint8_t	flags;
+			uint8_t	zero;
+		};
+	};
 	uint32_t	length;
 } __packed;
 
-#define RTR_MAX_VERSION		2
 #define RTR_MAX_PDU_SIZE	49152	/* XXX < IBUF_READ_SIZE */
 #define RTR_MAX_PDU_ERROR_SIZE	256
 #define RTR_DEFAULT_REFRESH	3600
@@ -104,15 +110,10 @@ struct rtr_routerkey {
 	/* followed by Subject Public Key Info */
 } __packed;
 
-#define FLAG_AFI_V6	0x1
-#define FLAG_AFI_MASK	FLAG_AFI_V6
 struct rtr_aspa {
 	struct rtr_header	hdr;
-	uint8_t			flags;
-	uint8_t			afi_flags;
-	uint16_t		cnt;
 	uint32_t		cas;
-	/* array of spas with cnt elements follows */
+	/* array of spas filling the rest of the packet */
 } __packed;
 
 struct rtr_endofdata {
@@ -188,7 +189,6 @@ struct rtr_session {
 	char				descr[PEER_DESCR_LEN];
 	struct roa_tree			roa_set;
 	struct aspa_tree		aspa;
-	struct aspa_tree		aspa_oldv6;
 	struct ibuf_read		r;
 	struct msgbuf			w;
 	struct timer_head		timers;
@@ -209,6 +209,9 @@ struct rtr_session {
 	char				last_recv_msg[REASON_LEN];
 	uint8_t				version;
 	uint8_t				prev_version;
+	uint8_t				min_version;
+	uint8_t				errored;
+
 };
 
 TAILQ_HEAD(, rtr_session) rtrs = TAILQ_HEAD_INITIALIZER(rtrs);
@@ -255,6 +258,14 @@ log_rtr_type(enum rtr_pdu_type type)
 	}
 };
 
+static uint8_t
+rtr_max_session_version(struct rtr_session *rs)
+{
+	if (rs->min_version > RTR_DEFAULT_VERSION)
+		return rs->min_version;
+	return RTR_DEFAULT_VERSION;
+}
+
 static void
 rtr_reset_cache(struct rtr_session *rs)
 {
@@ -263,7 +274,6 @@ rtr_reset_cache(struct rtr_session *rs)
 	timer_stop(&rs->timers, Timer_Rtr_Expire);
 	free_roatree(&rs->roa_set);
 	free_aspatree(&rs->aspa);
-	free_aspatree(&rs->aspa_oldv6);
 }
 
 static struct ibuf *
@@ -737,16 +747,26 @@ static int
 rtr_parse_aspa(struct rtr_session *rs, struct ibuf *pdu)
 {
 	struct rtr_aspa rtr_aspa;
-	struct aspa_tree *aspatree;
 	struct aspa_set *aspa, *a;
-	uint16_t cnt, i;
+	uint32_t cnt, i;
+	uint8_t flags;
 
 	if (ibuf_get(pdu, &rtr_aspa, sizeof(rtr_aspa)) == -1)
 		goto badlen;
 
-	cnt = ntohs(rtr_aspa.cnt);
-	if (ibuf_size(pdu) != cnt * sizeof(uint32_t))
-		goto badlen;
+	flags = rtr_aspa.hdr.flags;
+	cnt = ibuf_size(pdu) / sizeof(uint32_t);
+
+	if ((flags & FLAG_ANNOUNCE) && cnt == 0) {
+		rtr_send_error(rs, pdu, CORRUPT_DATA, "%s: "
+		    "announce with empty SPAS", log_rtr_type(ASPA));
+		return -1;
+	}
+	if ((flags & FLAG_ANNOUNCE) == 0 && cnt != 0) {
+		rtr_send_error(rs, pdu, CORRUPT_DATA, "%s: "
+		    "withdraw with non-empty SPAS", log_rtr_type(ASPA));
+		return -1;
+	}
 
 	if (rs->state != RTR_STATE_EXCHANGE) {
 		rtr_send_error(rs, pdu, CORRUPT_DATA, "%s: out of context",
@@ -754,12 +774,6 @@ rtr_parse_aspa(struct rtr_session *rs, struct ibuf *pdu)
 		return -1;
 	}
 
-	if (rtr_aspa.afi_flags & FLAG_AFI_V6) {
-		aspatree = &rs->aspa_oldv6;
-	} else {
-		aspatree = &rs->aspa;
-	}
-
 	/* treat ASPA records with too many SPAS like a withdraw */
 	if (cnt > MAX_ASPA_SPAS_COUNT) {
 		struct aspa_set needle = { 0 };
@@ -768,9 +782,9 @@ rtr_parse_aspa(struct rtr_session *rs, struct ibuf *pdu)
 		log_warnx("rtr %s: oversized ASPA PDU: "
 		    "imlicit withdraw of customerAS %s",
 		    log_rtr(rs), log_as(needle.as));
-		a = RB_FIND(aspa_tree, aspatree, &needle);
+		a = RB_FIND(aspa_tree, &rs->aspa, &needle);
 		if (a != NULL) {
-			RB_REMOVE(aspa_tree, aspatree, a);
+			RB_REMOVE(aspa_tree, &rs->aspa, a);
 			free_aspa(a);
 		}
 		return 0;
@@ -796,13 +810,13 @@ rtr_parse_aspa(struct rtr_session *rs, struct ibuf *pdu)
 		}
 	}
 
-	if (rtr_aspa.flags & FLAG_ANNOUNCE) {
-		a = RB_INSERT(aspa_tree, aspatree, aspa);
+	if (flags & FLAG_ANNOUNCE) {
+		a = RB_INSERT(aspa_tree, &rs->aspa, aspa);
 		if (a != NULL) {
-			RB_REMOVE(aspa_tree, aspatree, a);
+			RB_REMOVE(aspa_tree, &rs->aspa, a);
 			free_aspa(a);
 
-			if (RB_INSERT(aspa_tree, aspatree, aspa) != NULL) {
+			if (RB_INSERT(aspa_tree, &rs->aspa, aspa) != NULL) {
 				rtr_send_error(rs, NULL, INTERNAL_ERROR,
 				    "corrupt aspa tree");
 				free_aspa(aspa);
@@ -810,14 +824,14 @@ rtr_parse_aspa(struct rtr_session *rs, struct ibuf *pdu)
 			}
 		}
 	} else {
-		a = RB_FIND(aspa_tree, aspatree, aspa);
+		a = RB_FIND(aspa_tree, &rs->aspa, aspa);
 		if (a == NULL) {
 			rtr_send_error(rs, pdu, UNK_REC_WDRAWL, "%s %s",
 			    log_rtr_type(ASPA), log_aspa(aspa));
 			free_aspa(aspa);
 			return -1;
 		}
-		RB_REMOVE(aspa_tree, aspatree, a);
+		RB_REMOVE(aspa_tree, &rs->aspa, a);
 		free_aspa(a);
 		free_aspa(aspa);
 	}
@@ -954,7 +968,7 @@ rtr_parse_error(struct rtr_session *rs, struct ibuf *pdu)
 
 	if (ibuf_get(pdu, &rh, sizeof(rh)) == -1)
 		goto fail;
-	errcode = ntohs(rh.session_id);
+	errcode = ntohs(rh.errcode);
 
 	if (ibuf_get_n32(pdu, &pdu_len) == -1)
 		goto fail;
@@ -1087,13 +1101,14 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 
 	switch (event) {
 	case RTR_EVNT_UNSUPP_PROTO_VERSION:
-		if (rs->prev_version == rs->version) {
+		if (rs->prev_version == rs->version ||
+		    rs->version < rs->min_version) {
 			/*
 			 * Can't downgrade anymore, fail connection.
 			 * RFC requires sending the error with the
 			 * highest supported version number.
 			 */
-			rs->version = RTR_MAX_VERSION;
+			rs->version = rtr_max_session_version(rs);
 			rtr_send_error(rs, NULL, UNSUPP_PROTOCOL_VERS,
 			    "negotiation failed");
 			return;
@@ -1117,8 +1132,13 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 			rs->fd = -1;
 		}
 		/* try to reopen session */
-		timer_set(&rs->timers, Timer_Rtr_Retry,
-		    arc4random_uniform(10));
+		if (!rs->errored)
+			timer_set(&rs->timers, Timer_Rtr_Retry,
+			    arc4random_uniform(10));
+		else
+			timer_set(&rs->timers, Timer_Rtr_Retry, rs->retry);
+
+		rs->errored = 1;
 		/*
 		 * A close event during version negotiation needs to remain
 		 * in the negotiation state else the same error will happen
@@ -1193,6 +1213,7 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 		rtr_sem_release(rs->active_lock);
 		rtr_recalc();
 		rs->active_lock = 0;
+		rs->errored = 0;
 		/* clear the last errors */
 		rs->last_sent_error = NO_ERROR;
 		rs->last_recv_error = NO_ERROR;
@@ -1249,7 +1270,7 @@ rtr_dispatch_msg(struct pollfd *pfd, struct rtr_session *rs)
 		rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 		return;
 	}
-	if (pfd->revents & POLLOUT && rs->w.queued) {
+	if (pfd->revents & POLLOUT && msgbuf_queuelen(&rs->w) > 0) {
 		if ((error = ibuf_write(&rs->w)) == -1) {
 			if (errno != EAGAIN) {
 				log_warn("rtr %s: write error", log_rtr(rs));
@@ -1258,7 +1279,8 @@ rtr_dispatch_msg(struct pollfd *pfd, struct rtr_session *rs)
 		}
 		if (error == 0)
 			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
-		if (rs->w.queued == 0 && rs->state == RTR_STATE_ERROR)
+		if (rs->state == RTR_STATE_ERROR &&
+		    msgbuf_queuelen(&rs->w) == 0)
 			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 	}
 	if (pfd->revents & POLLIN) {
@@ -1364,7 +1386,7 @@ rtr_poll_events(struct pollfd *pfds, size_t npfds, time_t *timeout)
 		pfd->fd = rs->fd;
 		pfd->events = 0;
 
-		if (rs->w.queued)
+		if (msgbuf_queuelen(&rs->w) > 0)
 			pfd->events |= POLLOUT;
 		if (rs->state >= RTR_STATE_ESTABLISHED)
 			pfd->events |= POLLIN;
@@ -1374,24 +1396,24 @@ rtr_poll_events(struct pollfd *pfds, size_t npfds, time_t *timeout)
 }
 
 struct rtr_session *
-rtr_new(uint32_t id, char *descr)
+rtr_new(uint32_t id, struct rtr_config_msg *conf)
 {
 	struct rtr_session *rs;
 
 	if ((rs = calloc(1, sizeof(*rs))) == NULL)
-		fatal("RTR session %s", descr);
+		fatal("RTR session %s", conf->descr);
 
 	RB_INIT(&rs->roa_set);
 	RB_INIT(&rs->aspa);
-	RB_INIT(&rs->aspa_oldv6);
 	TAILQ_INIT(&rs->timers);
 	msgbuf_init(&rs->w);
 
-	strlcpy(rs->descr, descr, sizeof(rs->descr));
+	strlcpy(rs->descr, conf->descr, sizeof(rs->descr));
 	rs->id = id;
 	rs->session_id = -1;
-	rs->version = RTR_MAX_VERSION;
-	rs->prev_version = RTR_MAX_VERSION;
+	rs->min_version = conf->min_version;	/* must be set before version */
+	rs->version = rtr_max_session_version(rs);
+	rs->prev_version = rtr_max_session_version(rs);
 	rs->refresh = RTR_DEFAULT_REFRESH;
 	rs->retry = RTR_DEFAULT_RETRY;
 	rs->expire = RTR_DEFAULT_EXPIRE;
@@ -1444,8 +1466,8 @@ rtr_open(struct rtr_session *rs, int fd)
 	}
 
 	if (rs->state == RTR_STATE_CLOSED) {
-		rs->version = RTR_MAX_VERSION;
-		rs->prev_version = RTR_MAX_VERSION;
+		rs->version = rtr_max_session_version(rs);
+		rs->prev_version = rtr_max_session_version(rs);
 	}
 
 	rs->fd = rs->w.fd = fd;
@@ -1474,8 +1496,10 @@ rtr_config_merge(void)
 }
 
 void
-rtr_config_keep(struct rtr_session *rs)
+rtr_config_keep(struct rtr_session *rs, struct rtr_config_msg *conf)
 {
+	strlcpy(rs->descr, conf->descr, sizeof(rs->descr));
+	rs->min_version = conf->min_version;
 	rs->reconf_action = RECONF_KEEP;
 }
 
@@ -1500,8 +1524,6 @@ rtr_aspa_merge(struct aspa_tree *at)
 	TAILQ_FOREACH(rs, &rtrs, entry) {
 		RB_FOREACH(aspa, aspa_tree, &rs->aspa)
 			rtr_aspa_insert(at, aspa);
-		RB_FOREACH(aspa, aspa_tree, &rs->aspa_oldv6)
-			rtr_aspa_insert(at, aspa);
 	}
 }
 
@@ -1526,6 +1548,7 @@ rtr_show(struct rtr_session *rs, pid_t pid)
 
 	/* descr, remote_addr, local_addr and remote_port set by parent */
 	msg.version = rs->version;
+	msg.min_version = rs->min_version;
 	msg.serial = rs->serial;
 	msg.refresh = rs->refresh;
 	msg.retry = rs->retry;
